==> IncDTW_1.1.0.tar.gz: IncDTW/tests/testthat/test_rundtw_multiv.R <==
context("test running dtw")

goal <- function(h, x, ws, dm, sp){
   nx <- nrow(x)
   nh <- nrow(h)
   hnorm <- IncDTW::norm(h, type = "01")
   sapply(1:(nx-nh+1), function(i){
      y <- IncDTW::norm(x[i:(i+nh-1), , drop = F], type = "01")
      IncDTW::dtw2vec(y, hnorm, dist_method = dm,
                      step_pattern = sp, ws = ws)$dist
   })
}

goal_nonorm <- function(h, x, ws, dm, sp){
   nx <- nrow(x)
   nh <- nrow(h)
   sapply(1:(nx-nh+1), function(i){
      y <- x[i:(i+nh-1), , drop = F]
      IncDTW::dtw2vec(y, h, dist_method = dm,
                      step_pattern = sp, ws = ws)$dist
   })
}

noise <- function(i, nc) matrix(cumsum(rnorm(i*nc)), nrow =i, ncol=nc)

test_that("correct multivariate norm1", {
  
   dm <- "norm1"
   sp <- "symmetric1"
   nc <- 3
   WS <- 10
   
   h <- noise(10, nc)
   x <- rbind(noise(10, nc), h, noise(10, nc))
   
   # DO norm
   ist <- rundtw(Q = h, C = x, dist_method = dm, step_pattern = sp, 
                 ws = WS, normalize = TRUE, threshold = NULL, lower_bound = F)
   soll <- goal(h, x, WS, dm, sp)
   
   expect_equal(ist$dist, soll)
   
   # DONT norm
   ist <- rundtw(Q = h, C = x, dist_method = dm, step_pattern = sp, 
                 ws = WS, normalize = F, threshold = NULL, lower_bound = F)
   soll <- goal_nonorm(h, x, WS, dm, sp)
   
   expect_equal(ist$dist, soll)
   
})


test_that("correct multivariate norm2", {
   
   dm <- "norm2"
   sp <- "symmetric1"
   nc <- 3
   WS <- NULL

   h <- noise(10, nc)
   hnorm <- IncDTW::norm(h, type="01")
   x <- rbind(noise(10, nc), h, noise(10, nc))
   
   # DO norm
   ist <- rundtw(Q = h, C = x, dist_method = dm, step_pattern = sp, 
                 ws = WS, normalize = TRUE, threshold = NULL, lower_bound = F)
   soll <- goal(h, x, WS, dm, sp)
   
   expect_equal(ist$dist, soll)
   
   # DONT norm
   ist <- rundtw(Q = h, C = x, dist_method = dm, step_pattern = sp, 
                 ws = WS, normalize = F, threshold = NULL, lower_bound = F)
   soll <- goal_nonorm(h, x, WS, dm, sp)
   
   expect_equal(ist$dist, soll)
   
   
})


test_that("correct multivariate sym2", {
   
   dm <- "norm2"
   sp <- "symmetric2"
   nc <- 3
   WS <- 100
   
   h <- noise(10, nc)
   hnorm <- IncDTW::norm(h, type="01")
   x <- rbind(noise(10, nc), h, noise(10, nc))
   
   # DO norm
   ist <- rundtw(Q = h, C = x, dist_method = dm, step_pattern = sp, 
                 ws = WS, normalize = TRUE, threshold = NULL, lower_bound = F)
   soll <- goal(h, x, WS, dm, sp)
   
   expect_equal(ist$dist, soll)
   
   # DONT norm
   ist <- rundtw(Q = h, C = x, dist_method = dm, step_pattern = sp, 
                 ws = WS, normalize = F, threshold = NULL, lower_bound = F)
   soll <- goal_nonorm(h, x, WS, dm, sp)
   
   expect_equal(ist$dist, soll)
   
})


test_that("expected result, multivariate", {
   
   dm <- "norm1"
   sp <- "symmetric1"
   nc <- 3
   WS <- NULL
   deform <- function(x) (x + rnorm(1, 0, 100)) * abs(rnorm(1, 0, 100))
   
   h <- noise(10, nc)
   x <- rbind(noise(10, nc), h, noise(10, nc), deform(h), noise(10, nc), deform(h))
   soll <- c(11, 31, 51)
   
   # DONT lowerbound
   ist <- rundtw(Q = h, C = x, dist_method = dm, step_pattern = sp, normalize = TRUE,
                 threshold = NULL, lower_bound = FALSE)
   ist <- which(ist$dist < 10^(-10))
   
   expect_equal(ist, soll)
   
   
   # DO lowerbound
   ist <- rundtw(Q = h, C = x, dist_method = dm, step_pattern = sp, normalize = TRUE,
                 threshold = NULL, lower_bound = TRUE)
   ist <- which(ist$dist < 10^(-10))
   
   expect_equal(ist, soll)
   
})



test_that("kNN rundtw", {
   dm <- "norm1"
   sp <- "symmetric1"
   WS <- NULL
   nc <- 3
   goal_knn <- function(hnorm, x, sp, ws = 10, kNNk){
      allv <- rundtw(h, x, dist_method = dm, step_pattern = sp, ws = 10, 
                     threshold = NULL, k = 0, lower_bound = FALSE, normalize = TRUE)
      dd <- allv$dist
      best_i <- integer()
      for(i in 1:kNNk){
         ix <- which.min(dd)
         if(dd[ix] < Inf){
            best_i <- c(best_i, ix)
            dd[max(1, (1 + ix - nh)):min((ix + nh - 1), length(dd))] <- Inf   
         }
      }
      best_i
   }
   
   
   nx <- 500
   nh <- 20
   nfits <- 10
   
   nn <- nx - nfits * nh# nnoise
   nn <- nn/nfits
   
   h <- noise(nh, nc)
   hnorm <- IncDTW::norm( h , type="01")
   x <- noise(0,nc)
   for(i in 1:nfits){
      x <- rbind(x, noise(nn, nc), h)
   }
   
   # rundtw(hnorm, x,dm,  sp, k = 5, ws = 10, threshold = Inf,1,1,1)# DO norm
   # cpp_rundtw(hnorm, x, sp, ws = 10, threshold = Inf, kNNk = 10, do_norm = 1, use_ea = 1, use_lb = 1)
   
   k <- nfits + 2
   
   
   tmp <- rundtw(h, x, dist_method = dm, step_pattern = sp, ws = 10, threshold = Inf, k = k,
                 lower_bound = TRUE,normalize = TRUE)
   ist <- sort(tmp$knn_indices)
   
   soll <- sort( goal_knn(hnorm, x, sp, ws=10, kNNk = k) )
   
   
   expect_equal(ist, soll)
})





test_that("run time multiv", {
   skip("runtime comparison")
   
   dm <- "norm1"
   sp <- "symmetric1"
   WS <- NULL
   noise <- function(nr, nc) matrix(cumsum(rnorm(nr*nc)), nrow =nr, ncol=nc)
   # deform <- function(x) (x + rnorm(1, 0, 100)) * abs(rnorm(1, 0, 100))
   
   # speed comparison
   foo_2vec <- function(hnorm, x, ws){
      nh <- nrow(hnorm)
      nx <- nrow(x)
      sapply(1:(nx-nh+1), function(i){
         y <- IncDTW::norm(x[i:(i+nh-1), , drop=F], type = "01")
         IncDTW::dtw2vec(y, hnorm, dist_method = dm,
                         step_pattern = sp, ws = ws)$dist
      })
   }
   
   foo_cm <- function(hnorm, x, ws){
      nh <- nrow(hnorm)
      nx <- nrow(x)
      sapply(1:(nx-nh+1), function(i){
         y <- IncDTW::norm(x[i:(i+nh-1), , drop=F], type = "01")
         cm_tmp <- IncDTW::cm(y, hnorm, dist_method = dm, ws = ws)
         IncDTW::dtw2vec(cm_tmp, C = "cm", step_pattern = sp, ws = ws)$dist
      })
   }
   
   
   NH <- c(0.2, 0.3)
   NX <- c(100, 500, 750)
   
   # NH <- seq(0.1, 0.5, 0.05)
   # NX <- c(100, 200, 300, 500, 1000)
   NC <- c(2)
   
   df <- data.frame(expr = character(),
                    min = numeric(), 
                    mean = numeric(), 
                    median = numeric(), 
                    max = numeric(), 
                    neval = integer(), 
                    rel = numeric(), 
                    nh = integer(),
                    nx = integer(),
                    nfits = integer(),
                    nc = integer()
   )
   
   data.table::fwrite(df, file = "comp_time_test_multiv.txt")
   tic <- Sys.time()
   for(nhh in NH){
      print(nhh); print(Sys.time())
      
      for(nx in NX){
         print(c(nhh, nx)); print(Sys.time())
         
         for(nc in NC){
            print(c(nhh, nx, nc)); print(Sys.time())
            
            nh <- nhh*nx
            nfits0 <- 10
            nfits1 <-  min(nfits0, floor(nx/nh))
            
            nfits <- sample(nfits1, size = 1)
            
            nn <- nx - nfits * nh# nnoise
            nn <- nn/nfits
            
            h <- noise(nh, nc)
            hnorm <- IncDTW::norm( h , type="01")
            x <- matrix(numeric(), ncol=nc)
            for(i in 1:nfits){
               x <- rbind(x, noise(nn, nc), h)
            }
            
            
            
            mic <- microbenchmark::microbenchmark(foo_cm(hnorm, x, WS),
                                                  foo_2vec(hnorm, x, WS),
                                                  rundtw(hnorm, x, dm, sp, T, WS, NULL),
                                                  rundtw(hnorm, x, dm, sp, T, WS, Inf),
                                                  rundtw(hnorm, x, dm, sp, F, WS, NULL),
                                                  rundtw(hnorm, x, dm, sp, F, WS, Inf),
                                                  times = 10)
            df0 <- maxaR::rel_microbenchmark(mic, 
                                             cols = c("expr", "min", "mean", "median", "max", "neval"), 
                                             rel_expr = "rundtw(hnorm, x, dm, sp, T, WS, Inf)")$df
            df0$nh <- nh
            df0$nx <- nx
            df0$nfits <- nfits
            df0$nc <- nc
            
            data.table::fwrite(df0, file = "comp_time_test_multiv.txt", append = T, row.names = F, col.names = F)
            df <- rbind(df, df0)
         }
      }
   }
   print(Sys.time())
   
   require(ggplot2)
   
   df <- data.table::fread(file = "comp_time_test_multiv.txt")
   ggplot(df) + geom_line(aes(x = nh/nx, y = rel, group = expr, col = expr), size = 2)+
      facet_grid(~nx) + scale_y_log10()
   
   ggplot(df) + geom_line(aes(x = nh/nx, y = median, group = expr, col = expr), size = 2)+
      facet_grid(~nx) + scale_y_log10()
   
   df[, list(mean(median)), by = c( "nfits")]
   
})

==> IncDTW_1.1.1.tar.gz: IncDTW/tests/testthat/test_rundtw_multiv.R <==
context("test running dtw")

goal <- function(h, x, ws, dm, sp){
   nx <- nrow(x)
   nh <- nrow(h)
   hnorm <- IncDTW::norm(h, type = "01")
   sapply(1:(nx-nh+1), function(i){
      y <- IncDTW::norm(x[i:(i+nh-1), , drop = F], type = "01")
      IncDTW::dtw2vec(y, hnorm, dist_method = dm,
                      step_pattern = sp, ws = ws)$dist
   })
}

goal_nonorm <- function(h, x, ws, dm, sp){
   nx <- nrow(x)
   nh <- nrow(h)
   sapply(1:(nx-nh+1), function(i){
      y <- x[i:(i+nh-1), , drop = F]
      IncDTW::dtw2vec(y, h, dist_method = dm,
                      step_pattern = sp, ws = ws)$dist
   })
}

noise <- function(i, nc) matrix(cumsum(rnorm(i*nc)), nrow =i, ncol=nc)

test_that("correct multivariate norm1", {
  
   dm <- "norm1"
   sp <- "symmetric1"
   nc <- 3
   WS <- 10
   
   h <- noise(10, nc)
   x <- rbind(noise(10, nc), h, noise(10, nc))
   
   # DO norm
   ist <- rundtw(Q = h, C = x, dist_method = dm, step_pattern = sp, 
                 ws = WS, normalize = "01", threshold = NULL, lower_bound = F)
   soll <- goal(h, x, WS, dm, sp)
   
   expect_equal(ist$dist, soll)
   
   # DONT norm
   ist <- rundtw(Q = h, C = x, dist_method = dm, step_pattern = sp, 
                 ws = WS, normalize = "none", threshold = NULL, lower_bound = F)
   soll <- goal_nonorm(h, x, WS, dm, sp)
   
   expect_equal(ist$dist, soll)
   
})


test_that("correct multivariate norm2", {
   
   dm <- "norm2"
   sp <- "symmetric1"
   nc <- 3
   WS <- NULL

   h <- noise(10, nc)
   hnorm <- IncDTW::norm(h, type="01")
   x <- rbind(noise(10, nc), h, noise(10, nc))
   
   # DO norm
   ist <- rundtw(Q = h, C = x, dist_method = dm, step_pattern = sp, 
                 ws = WS, normalize = "01", threshold = NULL, lower_bound = F)
   soll <- goal(h, x, WS, dm, sp)
   
   expect_equal(ist$dist, soll)
   
   # DONT norm
   ist <- rundtw(Q = h, C = x, dist_method = dm, step_pattern = sp, 
                 ws = WS, normalize = "none", threshold = NULL, lower_bound = F)
   soll <- goal_nonorm(h, x, WS, dm, sp)
   
   expect_equal(ist$dist, soll)
   
   
})


test_that("correct multivariate sym2", {
   
   dm <- "norm2"
   sp <- "symmetric2"
   nc <- 3
   WS <- 100
   
   h <- noise(10, nc)
   hnorm <- IncDTW::norm(h, type="01")
   x <- rbind(noise(10, nc), h, noise(10, nc))
   
   # DO norm
   ist <- rundtw(Q = h, C = x, dist_method = dm, step_pattern = sp, 
                 ws = WS, normalize = "01", threshold = NULL, lower_bound = F)
   soll <- goal(h, x, WS, dm, sp)
   
   expect_equal(ist$dist, soll)
   
   # DONT norm
   ist <- rundtw(Q = h, C = x, dist_method = dm, step_pattern = sp, 
                 ws = WS, normalize = "none", threshold = NULL, lower_bound = F)
   soll <- goal_nonorm(h, x, WS, dm, sp)
   
   expect_equal(ist$dist, soll)
   
})


test_that("expected result, multivariate", {
   
   dm <- "norm1"
   sp <- "symmetric1"
   nc <- 3
   WS <- NULL
   deform <- function(x) (x + rnorm(1, 0, 100)) * abs(rnorm(1, 0, 100))
   
   h <- noise(10, nc)
   x <- rbind(noise(10, nc), h, noise(10, nc), deform(h), noise(10, nc), deform(h))
   soll <- c(11, 31, 51)
   
   # DONT lowerbound
   ist <- rundtw(Q = h, C = x, dist_method = dm, step_pattern = sp, normalize = "01",
                 threshold = NULL, lower_bound = FALSE)
   ist <- which(ist$dist < 10^(-10))
   
   expect_equal(ist, soll)
   
   
   # DO lowerbound
   ist <- rundtw(Q = h, C = x, dist_method = dm, step_pattern = sp, normalize = "01",
                 threshold = NULL, lower_bound = TRUE)
   ist <- which(ist$dist < 10^(-10))
   
   expect_equal(ist, soll)
   
})



test_that("kNN rundtw", {
   dm <- "norm1"
   sp <- "symmetric1"
   WS <- NULL
   nc <- 3
   goal_knn <- function(hnorm, x, sp, ws = 10, kNNk){
      allv <- rundtw(h, x, dist_method = dm, step_pattern = sp, ws = 10, 
                     threshold = NULL, k = 0, lower_bound = FALSE, normalize = "01")
      dd <- allv$dist
      best_i <- integer()
      for(i in 1:kNNk){
         ix <- which.min(dd)
         if(dd[ix] < Inf){
            best_i <- c(best_i, ix)
            dd[max(1, (1 + ix - nh)):min((ix + nh - 1), length(dd))] <- Inf   
         }
      }
      best_i
   }
   
   
   nx <- 500
   nh <- 20
   nfits <- 10
   
   nn <- nx - nfits * nh# nnoise
   nn <- nn/nfits
   
   h <- noise(nh, nc)
   hnorm <- IncDTW::norm( h , type="01")
   x <- noise(0,nc)
   for(i in 1:nfits){
      x <- rbind(x, noise(nn, nc), h)
   }
   
   # rundtw(hnorm, x,dm,  sp, k = 5, ws = 10, threshold = Inf,1,1,1)# DO norm
   # cpp_rundtw(hnorm, x, sp, ws = 10, threshold = Inf, kNNk = 10, do_norm = 1, use_ea = 1, use_lb = 1)
   
   k <- nfits + 2
   
   
   tmp <- rundtw(h, x, dist_method = dm, step_pattern = sp, ws = 10, threshold = Inf, k = k,
                 lower_bound = TRUE, normalize = "01")
   ist <- sort(tmp$knn_indices)
   
   soll <- sort( goal_knn(hnorm, x, sp, ws=10, kNNk = k) )
   
   
   expect_equal(ist, soll)
})





test_that("run time multiv", {
   skip("runtime comparison")
   
   dm <- "norm1"
   sp <- "symmetric1"
   WS <- NULL
   noise <- function(nr, nc) matrix(cumsum(rnorm(nr*nc)), nrow =nr, ncol=nc)
   # deform <- function(x) (x + rnorm(1, 0, 100)) * abs(rnorm(1, 0, 100))
   
   # speed comparison
   foo_2vec <- function(hnorm, x, ws){
      nh <- nrow(hnorm)
      nx <- nrow(x)
      sapply(1:(nx-nh+1), function(i){
         y <- IncDTW::norm(x[i:(i+nh-1), , drop=F], type = "01")
         IncDTW::dtw2vec(y, hnorm, dist_method = dm,
                         step_pattern = sp, ws = ws)$dist
      })
   }
   
   foo_cm <- function(hnorm, x, ws){
      nh <- nrow(hnorm)
      nx <- nrow(x)
      sapply(1:(nx-nh+1), function(i){
         y <- IncDTW::norm(x[i:(i+nh-1), , drop=F], type = "01")
         cm_tmp <- IncDTW::cm(y, hnorm, dist_method = dm, ws = ws)
         IncDTW::dtw2vec(cm_tmp, C = "cm", step_pattern = sp, ws = ws)$dist
      })
   }
   
   
   NH <- c(0.2, 0.3)
   NX <- c(100, 500, 750)
   
   # NH <- seq(0.1, 0.5, 0.05)
   # NX <- c(100, 200, 300, 500, 1000)
   NC <- c(2)
   
   df <- data.frame(expr = character(),
                    min = numeric(), 
                    mean = numeric(), 
                    median = numeric(), 
                    max = numeric(), 
                    neval = integer(), 
                    rel = numeric(), 
                    nh = integer(),
                    nx = integer(),
                    nfits = integer(),
                    nc = integer()
   )
   
   data.table::fwrite(df, file = "comp_time_test_multiv.txt")
   tic <- Sys.time()
   for(nhh in NH){
      print(nhh); print(Sys.time())
      
      for(nx in NX){
         print(c(nhh, nx)); print(Sys.time())
         
         for(nc in NC){
            print(c(nhh, nx, nc)); print(Sys.time())
            
            nh <- nhh*nx
            nfits0 <- 10
            nfits1 <-  min(nfits0, floor(nx/nh))
            
            nfits <- sample(nfits1, size = 1)
            
            nn <- nx - nfits * nh# nnoise
            nn <- nn/nfits
            
            h <- noise(nh, nc)
            hnorm <- IncDTW::norm( h , type="01")
            x <- matrix(numeric(), ncol=nc)
            for(i in 1:nfits){
               x <- rbind(x, noise(nn, nc), h)
            }
            
            
            
            mic <- microbenchmark::microbenchmark(foo_cm(hnorm, x, WS),
                                                  foo_2vec(hnorm, x, WS),
                                                  rundtw(hnorm, x, dm, sp, "01", WS, NULL),
                                                  rundtw(hnorm, x, dm, sp, "01", WS, Inf),
                                                  rundtw(hnorm, x, dm, sp, "none", WS, NULL),
                                                  rundtw(hnorm, x, dm, sp, "none", WS, Inf),
                                                  times = 10)
            df0 <- maxaR::rel_microbenchmark(mic, 
                                             cols = c("expr", "min", "mean", "median", "max", "neval"), 
                                             rel_expr = "rundtw(hnorm, x, dm, sp, T, WS, Inf)")$df
            df0$nh <- nh
            df0$nx <- nx
            df0$nfits <- nfits
            df0$nc <- nc
            
            data.table::fwrite(df0, file = "comp_time_test_multiv.txt", append = T, row.names = F, col.names = F)
            df <- rbind(df, df0)
         }
      }
   }
   print(Sys.time())
   
   require(ggplot2)
   
   df <- data.table::fread(file = "comp_time_test_multiv.txt")
   ggplot(df) + geom_line(aes(x = nh/nx, y = rel, group = expr, col = expr), size = 2)+
      facet_grid(~nx) + scale_y_log10()
   
   ggplot(df) + geom_line(aes(x = nh/nx, y = median, group = expr, col = expr), size = 2)+
      facet_grid(~nx) + scale_y_log10()
   
   df[, list(mean(median)), by = c( "nfits")]
   
})
